@@ -1,28 +1,37 @@
-/*
- * Copyright  2002 University of Southern California
+/* cairo - a vector graphics library with display and print output
  *
- * Permission to use, copy, modify, distribute, and sell this software
- * and its documentation for any purpose is hereby granted without
- * fee, provided that the above copyright notice appear in all copies
- * and that both that copyright notice and this permission notice
- * appear in supporting documentation, and that the name of the
- * University of Southern California not be used in advertising or
- * publicity pertaining to distribution of the software without
- * specific, written prior permission. The University of Southern
- * California makes no representations about the suitability of this
- * software for any purpose.  It is provided "as is" without express
- * or implied warranty.
+ * Copyright © 2002 University of Southern California
  *
- * THE UNIVERSITY OF SOUTHERN CALIFORNIA DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE UNIVERSITY OF
- * SOUTHERN CALIFORNIA BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
- * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * This library is free software; you can redistribute it and/or
+ * modify it either under the terms of the GNU Lesser General Public
+ * License version 2.1 as published by the Free Software Foundation
+ * (the "LGPL") or, at your option, under the terms of the Mozilla
+ * Public License Version 1.1 (the "MPL"). If you do not alter this
+ * notice, a recipient may use your version of this file under either
+ * the MPL or the LGPL.
  *
- * Author: Carl D. Worth <cworth@isi.edu>
+ * You should have received a copy of the LGPL along with this library
+ * in the file COPYING-LGPL-2.1; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * You should have received a copy of the MPL along with this library
+ * in the file COPYING-MPL-1.1
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
+ * OF ANY KIND, either express or implied. See the LGPL or the MPL for
+ * the specific language governing rights and limitations.
+ *
+ * The Original Code is the cairo graphics library.
+ *
+ * The Initial Developer of the Original Code is University of Southern
+ * California.
+ *
+ * Contributor(s):
+ *	Carl D. Worth <cworth@isi.edu>
  */
 
 #include "cairoint.h"
@@ -48,9 +57,9 @@ format_from_visual(XCBConnection *c, XCBVISUALID visual)
     if(!r)
 	return nil;
 
-    for(si = XCBRenderQueryPictFormatsScreens(r); si.rem; XCBRenderPICTSCREENNext(&si))
-	for(di = XCBRenderPICTSCREENDepths(si.data); di.rem; XCBRenderPICTDEPTHNext(&di))
-	    for(vi = XCBRenderPICTDEPTHVisuals(di.data); vi.rem; XCBRenderPICTVISUALNext(&vi))
+    for(si = XCBRenderQueryPictFormatsScreensIter(r); si.rem; XCBRenderPICTSCREENNext(&si))
+	for(di = XCBRenderPICTSCREENDepthsIter(si.data); di.rem; XCBRenderPICTDEPTHNext(&di))
+	    for(vi = XCBRenderPICTDEPTHVisualsIter(di.data); vi.rem; XCBRenderPICTVISUALNext(&vi))
 		if(vi.data->visual.id == visual.id)
 		{
 		    XCBRenderPICTFORMAT ret = vi.data->format;
@@ -123,7 +132,7 @@ format_from_cairo(XCBConnection *c, cairo_format_t fmt)
     if(!r)
 	return ret;
 
-    for(fi = XCBRenderQueryPictFormatsFormats(r); fi.rem; XCBRenderPICTFORMINFONext(&fi))
+    for(fi = XCBRenderQueryPictFormatsFormatsIter(r); fi.rem; XCBRenderPICTFORMINFONext(&fi))
     {
 	const XCBRenderDIRECTFORMAT *t, *f;
 	if(fi.data->type != XCBRenderPictTypeDirect)
@@ -231,6 +240,7 @@ _CAIRO_FORMAT_DEPTH (cairo_format_t format)
 static cairo_surface_t *
 _cairo_xcb_surface_create_similar (void		*abstract_src,
 				    cairo_format_t	format,
+				    int			drawable,
 				    int			width,
 				    int			height)
 {
@@ -314,7 +324,7 @@ static int
 bytes_per_line(XCBConnection *c, int width, int bpp)
 {
     int bitmap_pad = XCBGetSetup(c)->bitmap_format_scanline_pad;
-    return (bpp * width + bitmap_pad - 1) & -bitmap_pad;
+    return ((bpp * width + bitmap_pad - 1) & -bitmap_pad) >> 3;
 }
 
 static cairo_image_surface_t *
@@ -400,7 +410,7 @@ _cairo_xcb_surface_set_image (void			*abstract_surface,
 
     _cairo_xcb_surface_ensure_gc (surface);
     bpp = bits_per_pixel(surface->dpy, image->depth);
-    data_len = bytes_per_line(surface->dpy, surface->width, bpp) * surface->height;
+    data_len = bytes_per_line(surface->dpy, image->width, bpp) * image->height;
     XCBPutImage(surface->dpy, ZPixmap, surface->drawable, surface->gc,
 	      image->width,
 	      image->height,
@@ -475,7 +485,7 @@ _cairo_xcb_surface_set_filter (void *abstract_surface, cairo_filter_t filter)
     }
 
     XCBRenderSetPictureFilter(surface->dpy, surface->picture,
-			     strlen(render_filter), 0, render_filter, NULL);
+			     strlen(render_filter), render_filter, 0, NULL);
 
     return CAIRO_STATUS_SUCCESS;
 }
@@ -508,7 +518,7 @@ _cairo_xcb_surface_clone_similar (cairo_surface_t	*src,
     src_image = _cairo_surface_get_image (src);
 
     clone = (cairo_xcb_surface_t *)
-	_cairo_xcb_surface_create_similar (template, format,
+	_cairo_xcb_surface_create_similar (template, format, 0,
 					    src_image->width,
 					    src_image->height);
     if (clone == NULL)
@@ -720,7 +730,7 @@ _cairo_xcb_surface_create_pattern (void *abstract_surface,
     return CAIRO_INT_STATUS_UNSUPPORTED;
 }
 
-static const struct cairo_surface_backend cairo_xcb_surface_backend = {
+static const cairo_surface_backend_t cairo_xcb_surface_backend = {
     _cairo_xcb_surface_create_similar,
     _cairo_xcb_surface_destroy,
     _cairo_xcb_surface_pixels_per_inch,
@@ -735,7 +745,8 @@ static const struct cairo_surface_backend cairo_xcb_surface_backend = {
     _cairo_xcb_surface_copy_page,
     _cairo_xcb_surface_show_page,
     _cairo_xcb_surface_set_clip_region,
-    _cairo_xcb_surface_create_pattern
+    _cairo_xcb_surface_create_pattern,
+    NULL /* show_glyphs */
 };
 
 static void
